@@ -39,84 +39,8 @@ DirectivityShaperAudioProcessor::DirectivityShaperAudioProcessor()
 #endif
                   ),
 #endif
-parameters(*this, nullptr), oscParams (parameters)
+oscParams (parameters), parameters (*this, nullptr, "DirectivityShaper", createParameterLayout())
 {
-    oscParams.createAndAddParameter ("orderSetting", "Directivity Order", "",
-                                      NormalisableRange<float> (0.0f, 8.0f, 1.0f), 0.0f,
-                                      [](float value) {
-                                          if (value >= 0.5f && value < 1.5f) return "0th";
-                                          else if (value >= 1.5f && value < 2.5f) return "1st";
-                                          else if (value >= 2.5f && value < 3.5f) return "2nd";
-                                          else if (value >= 3.5f && value < 4.5f) return "3rd";
-                                          else if (value >= 4.5f && value < 5.5f) return "4th";
-                                          else if (value >= 5.5f && value < 6.5f) return "5th";
-                                          else if (value >= 6.5f && value < 7.5f) return "6th";
-                                          else if (value >= 7.5f) return "7th";
-                                          else return "Auto";},
-                                      nullptr);
-    oscParams.createAndAddParameter ("useSN3D", "Directivity Normalization", "",
-                                      NormalisableRange<float> (0.0f, 1.0f, 1.0f), 1.0f,
-                                      [](float value) { if (value >= 0.5f ) return "SN3D";
-                                          else return "N3D"; },
-                                      nullptr);
-
-    oscParams.createAndAddParameter("probeAzimuth", "probe Azimuth", CharPointer_UTF8 (R"(°)"),
-                                     NormalisableRange<float> (-180.0f, 180.0f, 0.01f), 0.0,
-                                     [](float value) {return String(value, 2);}, nullptr);
-    oscParams.createAndAddParameter("probeElevation", "probe Elevation", CharPointer_UTF8 (R"(°)"),
-                                     NormalisableRange<float> (-180.0f, 180.0f, 0.01f), 0.0,
-                                     [](float value) {return String(value, 2);}, nullptr);
-    oscParams.createAndAddParameter("probeRoll", "probe Roll", CharPointer_UTF8 (R"(°)"),
-                                     NormalisableRange<float> (-180.0f, 180.0f, 0.01f), 0.0,
-                                     [](float value) {return String(value, 2);}, nullptr);
-    oscParams.createAndAddParameter("probeLock", "Lock Directions", "",
-                                     NormalisableRange<float> (0.0f, 1.0f, 1.0f), 0.0,
-                                     [](float value) {return (value >= 0.5f) ? "locked" : "not locked";}, nullptr);
-    oscParams.createAndAddParameter("normalization", "Directivity Normalization", "",
-                                     NormalisableRange<float> (0.0f, 2.0f, 1.0f), 1.0,
-                                     [](float value) {
-                                         if (value >= 0.5f && value < 1.5f) return "on axis";
-                                         else if (value >= 1.5f && value < 2.5f) return "constant energy";
-                                         else return "basic decode";
-                                     }, nullptr);
-
-    for (int i = 0; i < numberOfBands; ++i)
-    {
-        oscParams.createAndAddParameter("filterType" + String(i), "Filter Type " + String(i+1), "",
-                                         NormalisableRange<float> (0.0f, 3.0f, 1.0f),  filterTypePresets[i],
-                                         [](float value) {
-                                             if (value >= 0.5f && value < 1.5f) return "Low-pass";
-                                             else if (value >= 1.5f && value < 2.5f) return "Band-pass";
-                                             else if (value >= 2.5f) return "High-pass";
-                                             else return "All-pass";},
-                                         nullptr);
-
-        oscParams.createAndAddParameter("filterFrequency" + String(i), "Filter Frequency " + String(i+1), "Hz",
-                                         NormalisableRange<float> (20.0f, 20000.0f, 1.0f, 0.4f), filterFrequencyPresets[i],
-                                         [](float value) { return String((int) value); }, nullptr);
-        oscParams.createAndAddParameter("filterQ" + String(i), "Filter Q " + String(i+1), "",
-                                         NormalisableRange<float> (0.05f, 10.0f, 0.05f), 0.5f,
-                                         [](float value) { return String(value, 2); },
-                                         nullptr);
-        oscParams.createAndAddParameter("filterGain" + String(i), "Filter Gain " + String(i+1), "dB",
-                                         NormalisableRange<float> (-60.0f, 10.0f, 0.1f), 0.0f,
-                                         [](float value) { return (value >= -59.9f) ? String(value, 1) : "-inf"; },
-                                         nullptr);
-        oscParams.createAndAddParameter("order" + String(i), "Order Band " + String(i+1), "",
-                                         NormalisableRange<float> (0.0f, 7.0f, 0.01f), 0.0,
-                                         [](float value) { return String(value, 2); }, nullptr);
-        oscParams.createAndAddParameter("shape" + String(i), "Shape Band " + String(i+1), "",
-                                         NormalisableRange<float> (0.0f, 1.0f, 0.01f), 0.0,
-                                         [](float value) { return String(value, 2); }, nullptr);
-        oscParams.createAndAddParameter("azimuth" + String(i), "Azimuth Band " + String(i+1), CharPointer_UTF8 (R"(°)"),
-                                         NormalisableRange<float> (-180.0f, 180.0f, 0.01f), 0.0,
-                                         [](float value) { return String(value, 2); }, nullptr);
-        oscParams.createAndAddParameter("elevation" + String(i), "Elevation Band " + String(i+1), CharPointer_UTF8 (R"(°)"),
-                                         NormalisableRange<float> (-180.0f, 180.0f, 0.01f), 0.0,
-                                         [](float value) {return String(value, 2);}, nullptr);
-    }
-
-
     orderSetting = parameters.getRawParameterValue ("orderSetting");
     useSN3D = parameters.getRawParameterValue ("useSN3D");
     probeAzimuth = parameters.getRawParameterValue ("probeAzimuth");
@@ -155,8 +79,6 @@ parameters(*this, nullptr), oscParams (parameters)
         probeGains[i] = 0.0f;
     }
 
-    parameters.state = ValueTree (Identifier ("DirectivityShaper"));
-
 
     FloatVectorOperations::clear(shOld[0], 64 * numberOfBands);
     FloatVectorOperations::clear(weights[0], 8 * numberOfBands);
@@ -315,19 +237,19 @@ void DirectivityShaperAudioProcessor::processBlock (AudioSampleBuffer& buffer, M
         }
     }
 
-    Weights::Normalization norm;
+    WeightsHelper::Normalization norm;
     if (*normalization < 0.5f)
-        norm = Weights::Normalization::BasicDecode;
+        norm = WeightsHelper::Normalization::BasicDecode;
     else if (*normalization >= 0.5f && *normalization < 1.5f)
-        norm = Weights::Normalization::OnAxis;
+        norm = WeightsHelper::Normalization::OnAxis;
     else
-        norm = Weights::Normalization::ConstantEnergy;
+        norm = WeightsHelper::Normalization::ConstantEnergy;
 
 
     for (int b = 0; b < numberOfBands; ++b)
     {
         float tempWeights[8];
-        const int nWeights = Weights::getWeights(*order[b], *shape[b], tempWeights);
+        const int nWeights = WeightsHelper::getWeights(*order[b], *shape[b], tempWeights);
 
         // fill higher orders with zeros
         for (int i = nWeights; i < 8; ++i)
@@ -338,11 +260,11 @@ void DirectivityShaperAudioProcessor::processBlock (AudioSampleBuffer& buffer, M
         FloatVectorOperations::copy(weights[b], tempWeights, 8);
 
         // normalize weights for GUI (7th order decode)
-        Weights::applyNormalization(weights[b], *order[b], 7, norm);
+        WeightsHelper::applyNormalization(weights[b], *order[b], 7, norm);
         // ============================================
 
         // normalize weights for audio
-        Weights::applyNormalization(tempWeights, *order[b], orderToWorkWith, norm, applySN3D);
+        WeightsHelper::applyNormalization(tempWeights, *order[b], orderToWorkWith, norm, applySN3D);
 
 
         Vector3D<float> pos = Conversions<float>::sphericalToCartesian(degreesToRadians(*azimuth[b]), degreesToRadians(*elevation[b]));
@@ -468,8 +390,8 @@ void DirectivityShaperAudioProcessor::parameterChanged (const String &parameterI
             {
                 iem::Quaternion<float> temp = probeQuat*quats[i];
                 temp.toYPR(ypr);
-                parameters.getParameterAsValue("azimuth" + String(i)).setValue(radiansToDegrees(ypr[0]));
-                parameters.getParameterAsValue("elevation" + String(i)).setValue(radiansToDegrees(ypr[1]));
+                parameters.getParameter ("azimuth" + String (i))->setValueNotifyingHost (parameters.getParameterRange ("azimuth" + String (i)).convertTo0to1 (radiansToDegrees (ypr[0])));
+                parameters.getParameter ("elevation" + String (i))->setValueNotifyingHost (parameters.getParameterRange ("elevation" + String (i)).convertTo0to1 (radiansToDegrees(ypr[1])));
             }
             moving = false;
             repaintSphere = true;
@@ -542,6 +464,115 @@ void DirectivityShaperAudioProcessor::oscMessageReceived (const OSCMessage &mess
     oscParams.processOSCMessage (msg);
 }
 
+void DirectivityShaperAudioProcessor::oscBundleReceived (const OSCBundle &bundle)
+{
+    for (int i = 0; i < bundle.size(); ++i)
+    {
+        auto elem = bundle[i];
+        if (elem.isMessage())
+            oscMessageReceived (elem.getMessage());
+        else if (elem.isBundle())
+            oscBundleReceived (elem.getBundle());
+    }
+}
+
+//==============================================================================
+AudioProcessorValueTreeState::ParameterLayout DirectivityShaperAudioProcessor::createParameterLayout()
+{
+    // add your audio parameters here
+    std::vector<std::unique_ptr<RangedAudioParameter>> params;
+
+    params.push_back (oscParams.createAndAddParameter ("orderSetting", "Directivity Order", "",
+                                     NormalisableRange<float> (0.0f, 8.0f, 1.0f), 0.0f,
+                                     [](float value) {
+                                         if (value >= 0.5f && value < 1.5f) return "0th";
+                                         else if (value >= 1.5f && value < 2.5f) return "1st";
+                                         else if (value >= 2.5f && value < 3.5f) return "2nd";
+                                         else if (value >= 3.5f && value < 4.5f) return "3rd";
+                                         else if (value >= 4.5f && value < 5.5f) return "4th";
+                                         else if (value >= 5.5f && value < 6.5f) return "5th";
+                                         else if (value >= 6.5f && value < 7.5f) return "6th";
+                                         else if (value >= 7.5f) return "7th";
+                                         else return "Auto";},
+                                     nullptr));
+
+    params.push_back (oscParams.createAndAddParameter ("useSN3D", "Directivity Normalization", "",
+                                     NormalisableRange<float> (0.0f, 1.0f, 1.0f), 1.0f,
+                                     [](float value) { if (value >= 0.5f ) return "SN3D";
+                                         else return "N3D"; },
+                                     nullptr));
+
+    params.push_back (oscParams.createAndAddParameter ("probeAzimuth", "probe Azimuth", CharPointer_UTF8 (R"(°)"),
+                                    NormalisableRange<float> (-180.0f, 180.0f, 0.01f), 0.0,
+                                    [](float value) {return String(value, 2);}, nullptr));
+
+    params.push_back (oscParams.createAndAddParameter ("probeElevation", "probe Elevation", CharPointer_UTF8 (R"(°)"),
+                                    NormalisableRange<float> (-180.0f, 180.0f, 0.01f), 0.0,
+                                    [](float value) {return String(value, 2);}, nullptr));
+
+    params.push_back (oscParams.createAndAddParameter ("probeRoll", "probe Roll", CharPointer_UTF8 (R"(°)"),
+                                    NormalisableRange<float> (-180.0f, 180.0f, 0.01f), 0.0,
+                                    [](float value) {return String(value, 2);}, nullptr));
+
+    params.push_back (oscParams.createAndAddParameter ("probeLock", "Lock Directions", "",
+                                    NormalisableRange<float> (0.0f, 1.0f, 1.0f), 0.0,
+                                    [](float value) {return (value >= 0.5f) ? "locked" : "not locked";}, nullptr));
+
+    params.push_back (oscParams.createAndAddParameter ("normalization", "Directivity Normalization", "",
+                                    NormalisableRange<float> (0.0f, 2.0f, 1.0f), 1.0,
+                                    [](float value) {
+                                        if (value >= 0.5f && value < 1.5f) return "on axis";
+                                        else if (value >= 1.5f && value < 2.5f) return "constant energy";
+                                        else return "basic decode";
+                                    }, nullptr));
+
+    for (int i = 0; i < numberOfBands; ++i)
+    {
+        params.push_back (oscParams.createAndAddParameter ("filterType" + String(i), "Filter Type " + String(i+1), "",
+                                        NormalisableRange<float> (0.0f, 3.0f, 1.0f),  filterTypePresets[i],
+                                        [](float value) {
+                                            if (value >= 0.5f && value < 1.5f) return "Low-pass";
+                                            else if (value >= 1.5f && value < 2.5f) return "Band-pass";
+                                            else if (value >= 2.5f) return "High-pass";
+                                            else return "All-pass";},
+                                        nullptr));
+
+        params.push_back (oscParams.createAndAddParameter ("filterFrequency" + String(i), "Filter Frequency " + String(i+1), "Hz",
+                                        NormalisableRange<float> (20.0f, 20000.0f, 1.0f, 0.4f), filterFrequencyPresets[i],
+                                        [](float value) { return String((int) value); }, nullptr));
+
+        params.push_back (oscParams.createAndAddParameter ("filterQ" + String(i), "Filter Q " + String(i+1), "",
+                                        NormalisableRange<float> (0.05f, 10.0f, 0.05f), 0.5f,
+                                        [](float value) { return String(value, 2); },
+                                        nullptr));
+
+        params.push_back (oscParams.createAndAddParameter ("filterGain" + String(i), "Filter Gain " + String(i+1), "dB",
+                                        NormalisableRange<float> (-60.0f, 10.0f, 0.1f), 0.0f,
+                                        [](float value) { return (value >= -59.9f) ? String(value, 1) : "-inf"; },
+                                        nullptr));
+
+        params.push_back (oscParams.createAndAddParameter ("order" + String(i), "Order Band " + String(i+1), "",
+                                        NormalisableRange<float> (0.0f, 7.0f, 0.01f), 0.0,
+                                        [](float value) { return String(value, 2); }, nullptr));
+
+        params.push_back (oscParams.createAndAddParameter ("shape" + String(i), "Shape Band " + String(i+1), "",
+                                        NormalisableRange<float> (0.0f, 1.0f, 0.01f), 0.0,
+                                        [](float value) { return String(value, 2); }, nullptr));
+
+        params.push_back (oscParams.createAndAddParameter ("azimuth" + String(i), "Azimuth Band " + String(i+1), CharPointer_UTF8 (R"(°)"),
+                                        NormalisableRange<float> (-180.0f, 180.0f, 0.01f), 0.0,
+                                        [](float value) { return String(value, 2); }, nullptr));
+
+        params.push_back (oscParams.createAndAddParameter ("elevation" + String(i), "Elevation Band " + String(i+1), CharPointer_UTF8 (R"(°)"),
+                                        NormalisableRange<float> (-180.0f, 180.0f, 0.01f), 0.0,
+                                        [](float value) {return String(value, 2);}, nullptr));
+    }
+
+
+
+    return { params.begin(), params.end() };
+}
+
 //==============================================================================
 // This creates new instances of the plugin..
 AudioProcessor* JUCE_CALLTYPE createPluginFilter()
