@@ -36,43 +36,15 @@ EnergyVisualizerAudioProcessor::EnergyVisualizerAudioProcessor()
                      #endif
                        ),
 #endif
-parameters(*this, nullptr), oscParams (parameters)
+oscParams (parameters), parameters (*this, nullptr, "EnergyVisualizer", createParameterLayout())
 {
-    oscParams.createAndAddParameter ("orderSetting", "Ambisonics Order", "",
-                                      NormalisableRange<float> (0.0f, 8.0f, 1.0f), 0.0f,
-                                      [](float value)
-                                      {
-                                          if (value >= 0.5f && value < 1.5f) return "0th";
-                                          else if (value >= 1.5f && value < 2.5f) return "1st";
-                                          else if (value >= 2.5f && value < 3.5f) return "2nd";
-                                          else if (value >= 3.5f && value < 4.5f) return "3rd";
-                                          else if (value >= 4.5f && value < 5.5f) return "4th";
-                                          else if (value >= 5.5f && value < 6.5f) return "5th";
-                                          else if (value >= 6.5f && value < 7.5f) return "6th";
-                                          else if (value >= 7.5f) return "7th";
-                                          else return "Auto";
-                                      }, nullptr);
-
-    oscParams.createAndAddParameter ("useSN3D", "Normalization", "",
-                                      NormalisableRange<float> (0.0f, 1.0f, 1.0f), 1.0f,
-                                      [](float value)
-                                      {
-                                          if (value >= 0.5f ) return "SN3D";
-                                          else return "N3D";
-                                      }, nullptr);
-
-    oscParams.createAndAddParameter("peakLevel", "Peak level", "dB",
-                                     NormalisableRange<float> (-50.0f, 10.0f, 0.1f), 0.0,
-                                     [](float value) {return String(value, 1);}, nullptr);
-
-
     orderSetting = parameters.getRawParameterValue ("orderSetting");
     useSN3D = parameters.getRawParameterValue ("useSN3D");
     peakLevel = parameters.getRawParameterValue ("peakLevel");
+    dynamicRange = parameters.getRawParameterValue ("dynamicRange");
 
     parameters.addParameterListener ("orderSetting", this);
 
-    parameters.state = ValueTree (Identifier ("EnergyVisualizer"));
 
     Eigen::Matrix<float,64,nSamplePoints> Y;
     // calc Y and YH
@@ -88,6 +60,8 @@ parameters(*this, nullptr), oscParams (parameters)
     rms.resize(nSamplePoints);
 
     oscReceiver.addListener (this);
+
+    startTimer (200);
 }
 
 EnergyVisualizerAudioProcessor::~EnergyVisualizerAudioProcessor()
@@ -184,6 +158,9 @@ void EnergyVisualizerAudioProcessor::processBlock (AudioSampleBuffer& buffer, Mi
 
     checkInputAndOutput(this, *orderSetting, 0);
 
+    if (! doProcessing.get())
+        return;
+
     //const int nCh = buffer.getNumChannels();
     const int L = buffer.getNumSamples();
     const int workingOrder = jmin(isqrt(buffer.getNumChannels())-1, input.getOrder());
@@ -214,7 +191,7 @@ void EnergyVisualizerAudioProcessor::processBlock (AudioSampleBuffer& buffer, Mi
     float oneMinusTimeConstant = 1.0f - timeConstant;
     for (int i = 0; i < nSamplePoints; ++i)
     {
-        pRms[i] = timeConstant * pRms[i] + oneMinusTimeConstant * ((Decibels::gainToDecibels(sampledSignals.getRMSLevel(i, 0, L)) - *peakLevel) / 35.0f + 1.0f);
+        pRms[i] = timeConstant * pRms[i] + oneMinusTimeConstant * ((Decibels::gainToDecibels(sampledSignals.getRMSLevel(i, 0, L)) - *peakLevel) / *dynamicRange + 1.0f);
     }
     FloatVectorOperations::clip(pRms, rms.getRawDataPointer(), 0.0f, 1.0f, nSamplePoints);
 }
@@ -284,6 +261,70 @@ void EnergyVisualizerAudioProcessor::oscMessageReceived (const OSCMessage &messa
     oscParams.processOSCMessage (msg);
 }
 
+void EnergyVisualizerAudioProcessor::oscBundleReceived (const OSCBundle &bundle)
+{
+    for (int i = 0; i < bundle.size(); ++i)
+    {
+        auto elem = bundle[i];
+        if (elem.isMessage())
+            oscMessageReceived (elem.getMessage());
+        else if (elem.isBundle())
+            oscBundleReceived (elem.getBundle());
+    }
+}
+
+//==============================================================================
+AudioProcessorValueTreeState::ParameterLayout EnergyVisualizerAudioProcessor::createParameterLayout()
+{
+    // add your audio parameters here
+    std::vector<std::unique_ptr<RangedAudioParameter>> params;
+
+    params.push_back (oscParams.createAndAddParameter ("orderSetting", "Ambisonics Order", "",
+                                     NormalisableRange<float> (0.0f, 8.0f, 1.0f), 0.0f,
+                                     [](float value)
+                                     {
+                                         if (value >= 0.5f && value < 1.5f) return "0th";
+                                         else if (value >= 1.5f && value < 2.5f) return "1st";
+                                         else if (value >= 2.5f && value < 3.5f) return "2nd";
+                                         else if (value >= 3.5f && value < 4.5f) return "3rd";
+                                         else if (value >= 4.5f && value < 5.5f) return "4th";
+                                         else if (value >= 5.5f && value < 6.5f) return "5th";
+                                         else if (value >= 6.5f && value < 7.5f) return "6th";
+                                         else if (value >= 7.5f) return "7th";
+                                         else return "Auto";
+                                     }, nullptr));
+
+    params.push_back (oscParams.createAndAddParameter ("useSN3D", "Normalization", "",
+                                     NormalisableRange<float> (0.0f, 1.0f, 1.0f), 1.0f,
+                                     [](float value)
+                                     {
+                                         if (value >= 0.5f ) return "SN3D";
+                                         else return "N3D";
+                                     }, nullptr));
+
+    params.push_back (oscParams.createAndAddParameter ("peakLevel", "Peak level", "dB",
+                                    NormalisableRange<float> (-50.0f, 10.0f, 0.1f), 0.0,
+                                    [](float value) {return String(value, 1);}, nullptr));
+
+    params.push_back (oscParams.createAndAddParameter ("dynamicRange", "Dynamic Range", "dB",
+                                                       NormalisableRange<float> (10.0f, 60.0f, 1.f), 35.0,
+                                                       [](float value) {return String (value, 0);}, nullptr));
+
+    return { params.begin(), params.end() };
+}
+
+
+//==============================================================================
+void EnergyVisualizerAudioProcessor::timerCallback()
+{
+    RelativeTime timeDifference = Time::getCurrentTime() - lastEditorTime.get();
+    if (timeDifference.inMilliseconds() > 800)
+        doProcessing = false;
+    else
+        doProcessing = true;
+}
+
+
 //==============================================================================
 // This creates new instances of the plugin..
 AudioProcessor* JUCE_CALLTYPE createPluginFilter()
