@@ -86,31 +86,34 @@ public:
 
     //======== OSC =================================================================
     void oscMessageReceived (const OSCMessage &message) override;
+    void oscBundleReceived (const OSCBundle &bundle) override;
     OSCReceiverPlus& getOSCReceiver () { return oscReceiver; }
     //==============================================================================
 
+    //======= Parameters ===========================================================
+    AudioProcessorValueTreeState::ParameterLayout createParameterLayout();
+    //==============================================================================
+
 
 //==============================================================================
-    // parameters/functions for interfacing with GUI
-    AudioProcessorValueTreeState parameters;
-
-    void setNetworkOrder (int order);
 
     int maxPossibleChannels = 64;
 
     void setFreezeMode (bool freezeState);
     void updateFilterParameters ();
-        void getT60ForFrequencyArray(double* frequencies, double* t60Data, size_t numSamples);
+    void getT60ForFrequencyArray (double* frequencies, double* t60Data, size_t numSamples);
 
 private:
-//==============================================================================
-    JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR (FdnReverbAudioProcessor)
-    
+    //==============================================================================
     OSCParameterInterface oscParams;
     OSCReceiverPlus oscReceiver;
+    AudioProcessorValueTreeState parameters;
+
+	AudioBuffer<float> copyBuffer;
 
     // parameters (from GUI)
     float *revTime;
+	float *fadeInTime;
     float *delayLength;
 
     float *highCutoff;
@@ -119,9 +122,10 @@ private:
     float *lowCutoff;
     float *lowQ;
     float *lowGain;
-    //float *fdnSize;
     float *wet;
 
 
-    FeedbackDelayNetwork fdn;
+    FeedbackDelayNetwork fdn, fdnFade;
+
+    JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR (FdnReverbAudioProcessor)
 };
