@@ -23,14 +23,9 @@
 #include "PluginProcessor.h"
 #include "PluginEditor.h"
 
-#ifndef M_PI
-#define M_PI 3.14159265358979323846
-#endif
-
 //==============================================================================
 RoomEncoderAudioProcessorEditor::RoomEncoderAudioProcessorEditor (RoomEncoderAudioProcessor& p, AudioProcessorValueTreeState& vts)
-: AudioProcessorEditor (&p),
-processor (p), valueTreeState(vts), footer (p.getOSCReceiver()),
+: AudioProcessorEditor (&p), footer (p.getOSCReceiver()), processor (p), valueTreeState(vts),
 sourceElement(*valueTreeState.getParameter("sourceX"), valueTreeState.getParameterRange("sourceX"),
               *valueTreeState.getParameter("sourceY"), valueTreeState.getParameterRange("sourceY"),
               *valueTreeState.getParameter("sourceZ"), valueTreeState.getParameterRange("sourceZ")),
@@ -177,7 +172,7 @@ listenerElement(*valueTreeState.getParameter("listenerX"), valueTreeState.getPar
     slListenerX.setSliderStyle (Slider::RotaryHorizontalVerticalDrag);
     slListenerX.setTextBoxStyle (Slider::TextBoxBelow, false, 50, 15);
     slListenerX.setColour (Slider::rotarySliderOutlineColourId, globalLaF.ClWidgetColours[1]);
-    slListenerX.setRotaryParameters(M_PI, 3*M_PI, false);
+    slListenerX.setRotaryParameters(MathConstants<float>::pi, 3*MathConstants<float>::pi, false);
     slListenerX.setTextValueSuffix(" m");
     slListenerX.setTooltip("listener position x");
     slListenerX.addListener(this);
@@ -262,8 +257,8 @@ listenerElement(*valueTreeState.getParameter("listenerX"), valueTreeState.getPar
     slHighShelfGain.setColour (Slider::rotarySliderOutlineColourId, Colours::orangered);
 
     addAndMakeVisible(&fv);
-    fv.addCoefficients(&processor.lowShelfCoefficients, Colours::cyan, &slLowShelfFreq, &slLowShelfGain);
-    fv.addCoefficients(&processor.highShelfCoefficients, Colours::orangered, &slHighShelfFreq, &slHighShelfGain);
+    fv.addCoefficients(processor.lowShelfCoefficients, Colours::cyan, &slLowShelfFreq, &slLowShelfGain);
+    fv.addCoefficients(processor.highShelfCoefficients, Colours::orangered, &slHighShelfFreq, &slHighShelfGain);
 
     addAndMakeVisible(&rv);
     rv.setDataPointers(p.allGains, p.mRadius, p.numRefl);
