@@ -34,139 +34,8 @@ RoomEncoderAudioProcessor::RoomEncoderAudioProcessor()
 #endif
                   ),
 #endif
-parameters (*this, nullptr), oscParams (parameters)
+oscParams (parameters), parameters (*this, nullptr, "RoomEncoder", createParameterLayout())
 {
-    oscParams.createAndAddParameter ("directivityOrderSetting", "Input Directivity Order", "",
-                                      NormalisableRange<float> (0.0f, 8.0f, 1.0f), 1.0f,
-                                      [](float value) {
-                                          if (value >= 0.5f && value < 1.5f) return "0th";
-                                          else if (value >= 1.5f && value < 2.5f) return "1st";
-                                          else if (value >= 2.5f && value < 3.5f) return "2nd";
-                                          else if (value >= 3.5f && value < 4.5f) return "3rd";
-                                          else if (value >= 4.5f && value < 5.5f) return "4th";
-                                          else if (value >= 5.5f && value < 6.5f) return "5th";
-                                          else if (value >= 6.5f && value < 7.5f) return "6th";
-                                          else if (value >= 7.5f) return "7th";
-                                          else return "Auto"; },
-                                      nullptr);
-    oscParams.createAndAddParameter ("inputIsSN3D", "Input Directivity Normalization", "",
-                                      NormalisableRange<float> (0.0f, 1.0f, 1.0f), 1.0f,
-                                      [](float value) { if (value >= 0.5f ) return "SN3D";
-                                          else return "N3D"; },
-                                      nullptr);
-
-    oscParams.createAndAddParameter ("orderSetting", "Output Ambisonics Order", "",
-                                      NormalisableRange<float> (0.0f, 8.0f, 1.0f), 0.0f,
-                                      [](float value) {
-                                          if (value >= 0.5f && value < 1.5f) return "0th";
-                                          else if (value >= 1.5f && value < 2.5f) return "1st";
-                                          else if (value >= 2.5f && value < 3.5f) return "2nd";
-                                          else if (value >= 3.5f && value < 4.5f) return "3rd";
-                                          else if (value >= 4.5f && value < 5.5f) return "4th";
-                                          else if (value >= 5.5f && value < 6.5f) return "5th";
-                                          else if (value >= 6.5f && value < 7.5f) return "6th";
-                                          else if (value >= 7.5f) return "7th";
-                                          else return "Auto"; },
-                                      nullptr);
-    oscParams.createAndAddParameter ("useSN3D", "Normalization", "",
-                                      NormalisableRange<float> (0.0f, 1.0f, 1.0f), 1.0f,
-                                      [](float value) { if (value >= 0.5f ) return "SN3D";
-                                          else return "N3D"; },
-                                      nullptr);
-
-    oscParams.createAndAddParameter ("roomX", "room size x", "m",
-                                     NormalisableRange<float> (1.0f, 30.0f, 0.01f), 10.0f,
-                                     [](float value) { return String(value, 2); }, nullptr);
-    oscParams.createAndAddParameter ("roomY", "room size y", "m",
-                                     NormalisableRange<float> (1.0f, 30.0f, 0.01f), 11.0f,
-                                     [](float value) { return String(value, 2); }, nullptr);
-    oscParams.createAndAddParameter ("roomZ", "room size z", "m",
-                                     NormalisableRange<float> (1.0f, 20.0f, 0.01f), 7.0f,
-                                     [](float value) { return String(value, 2); }, nullptr);
-
-    oscParams.createAndAddParameter ("sourceX", "source position x", "m",
-                                     NormalisableRange<float> (-15.0f, 15.0f, 0.001f), 1.0f,
-                                     [](float value) { return String(value, 3); }, nullptr);
-    oscParams.createAndAddParameter ("sourceY", "source position y", "m",
-                                     NormalisableRange<float> (-15.0f, 15.0f, 0.001f), 1.0f,
-                                     [](float value) { return String(value, 3); }, nullptr);
-    oscParams.createAndAddParameter ("sourceZ", "source position z", "m",
-                                     NormalisableRange<float> (-10.0f, 10.0f, 0.001f), -1.0f,
-                                     [](float value) { return String(value, 3); }, nullptr);
-
-    oscParams.createAndAddParameter ("listenerX", "listener position x", "m",
-                                     NormalisableRange<float> (-15.0f, 15.0f, 0.001f), -1.0f,
-                                     [](float value) { return String(value, 3); }, nullptr);
-    oscParams.createAndAddParameter ("listenerY", "listener position y", "m",
-                                     NormalisableRange<float> (-15.0f, 15.0f, 0.001f), -1.0f,
-                                     [](float value) { return String(value, 3); }, nullptr);
-    oscParams.createAndAddParameter ("listenerZ", "listener position z", "m",
-                                     NormalisableRange<float> (-10.0f, 10.0f, 0.001f), -1.0f,
-                                     [](float value) { return String(value, 3); }, nullptr);
-
-    oscParams.createAndAddParameter ("numRefl", "number of reflections", "",
-                                     NormalisableRange<float> (0.0f, nImgSrc-1, 1.0f), 33.0f,
-                                     [](float value) { return String((int) value); }, nullptr);
-
-    oscParams.createAndAddParameter ("lowShelfFreq", "LowShelf Frequency", "Hz",
-                                     NormalisableRange<float> (20.0f, 20000.0f, 1.0f, 0.2f), 100.0,
-                                     [](float value) { return String((int) value); }, nullptr);
-    oscParams.createAndAddParameter ("lowShelfGain", "LowShelf Gain", "dB",
-                                     NormalisableRange<float> (-15.0f, 5.0f, 0.1f), -5.0f,
-                                     [](float value) { return String(value, 1); }, nullptr);
-
-    oscParams.createAndAddParameter ("highShelfFreq", "HighShelf Frequency", "Hz",
-                                     NormalisableRange<float> (20., 20000.0f, 1.0f, 0.2f), 8000.0,
-                                     [](float value) { return String((int) value); }, nullptr);
-    oscParams.createAndAddParameter ("highShelfGain", "HighShelf Gain", "dB",
-                                     NormalisableRange<float> (-15.0f, 5.0f, 0.1f), -5.0f,
-                                     [](float value) { return String(value, 1); }, nullptr);
-
-    oscParams.createAndAddParameter ("reflCoeff", "Reflection Coefficient", "dB",
-                                     NormalisableRange<float> (-15.0f, 0.0f, 0.01f), -1.0f,
-                                     [](float value) { return String(value, 2); }, nullptr);
-
-    oscParams.createAndAddParameter ("syncChannel", "Synchronize to Channel", "",
-                                      NormalisableRange<float> (0.0f, 4.0f, 1.0f), 0.0f,
-                                      [](float value) {
-                                          if (value >= 0.5f && value < 1.5f) return "Channel 1";
-                                          else if (value >= 1.5f && value < 2.5f) return "Channel 2";
-                                          else if (value >= 2.5f && value < 3.5f) return "Channel 3";
-                                          else if (value >= 3.5f) return "Channel 4";
-                                          else return "None"; },
-                                      nullptr);
-
-    oscParams.createAndAddParameter ("syncRoomSize", "Synchronize Room Dimensions", "",
-                                      NormalisableRange<float> (0.0f, 1.0f, 1.0f), 1.0f,
-                                      [](float value) {
-                                          if (value >= 0.5f) return "YES";
-                                          else return "NO"; },
-                                      nullptr);
-
-    oscParams.createAndAddParameter ("syncReflection", "Synchronize Reflection Properties", "",
-                                      NormalisableRange<float> (0.0f, 1.0f, 1.0f), 1.0f,
-                                      [](float value) {
-                                          if (value >= 0.5f) return "YES";
-                                          else return "NO"; },
-                                      nullptr);
-
-    oscParams.createAndAddParameter ("syncListener", "Synchronize Listener Position", "",
-                                      NormalisableRange<float> (0.0f, 1.0f, 1.0f), 1.0f,
-                                      [](float value) {
-                                          if (value >= 0.5f) return "YES";
-                                          else return "NO"; },
-                                      nullptr);
-
-    oscParams.createAndAddParameter ("renderDirectPath", "Render Direct Path", "",
-                                      NormalisableRange<float> (0.0f, 1.0f, 1.0f), 1.0f,
-                                      [](float value) {
-                                          if (value >= 0.5f) return "YES";
-                                          else return "NO"; },
-                                      nullptr);
-
-
-    parameters.state = ValueTree (Identifier ("RoomEncoder"));
-
     directivityOrderSetting = parameters.getRawParameterValue ("directivityOrderSetting");
     inputIsSN3D = parameters.getRawParameterValue ("inputIsSN3D");
     orderSetting = parameters.getRawParameterValue ("orderSetting");
@@ -785,19 +654,31 @@ void RoomEncoderAudioProcessor::processBlock (AudioSampleBuffer& buffer, MidiBuf
     for (int q = workingNumRefl + 1; q < nImgSrc; ++q)
         oldDelay[q] = mRadius[q]*dist2smpls;
 
-    // Read from buffer
+    // ======= Read from delayBuffer and clear read content ==============
     buffer.clear();
+
+    const int blockSize1 = jmin (bufferSize - readOffset, L);
+
     for (int channel = 0; channel < maxNChOut; ++channel)
     {
-        buffer.copyFrom(channel, 0, delayBufferWritePtrArray[channel] + readOffset, L);
+        buffer.copyFrom(channel, 0, delayBufferWritePtrArray[channel] + readOffset, blockSize1);
+        delayBuffer.clear(channel, readOffset, blockSize1);
+    }
 
-        delayBuffer.clear(channel, readOffset, L);
+    const int numLeft = L - blockSize1;
+    if (numLeft > 0)
+    {
+        for (int channel = 0; channel < maxNChOut; ++channel)
+        {
+            buffer.copyFrom(channel, 0, delayBufferWritePtrArray[channel], numLeft);
+            delayBuffer.clear(channel, 0, numLeft);
+        }
     }
 
     _numRefl = currNumRefl;
 
     readOffset += L;
-    if (readOffset >= bufferSize) readOffset = 0;
+    if (readOffset >= bufferSize) readOffset -= bufferSize;
 
 }
 
@@ -851,9 +732,9 @@ void RoomEncoderAudioProcessor::timerCallback()
         {
             if (roomParam.validRoomData) {
                 readingSharedParams = true;
-                parameters.getParameterAsValue("roomX").setValue(roomParam.roomX);
-                parameters.getParameterAsValue("roomY").setValue(roomParam.roomY);
-                parameters.getParameterAsValue("roomZ").setValue(roomParam.roomZ);
+                parameters.getParameter ("roomX")->setValueNotifyingHost (parameters.getParameterRange ("roomX").convertTo0to1 (roomParam.roomX));
+                parameters.getParameter ("roomY")->setValueNotifyingHost (parameters.getParameterRange ("roomY").convertTo0to1 (roomParam.roomY));
+                parameters.getParameter ("roomZ")->setValueNotifyingHost (parameters.getParameterRange ("roomZ").convertTo0to1 (roomParam.roomZ));
                 readingSharedParams = false;
             }
             else
@@ -868,9 +749,9 @@ void RoomEncoderAudioProcessor::timerCallback()
         {
             if (roomParam.validListenerData) {
                 readingSharedParams = true;
-                parameters.getParameterAsValue("listenerX").setValue(roomParam.listenerX);
-                parameters.getParameterAsValue("listenerY").setValue(roomParam.listenerY);
-                parameters.getParameterAsValue("listenerZ").setValue(roomParam.listenerZ);
+                parameters.getParameter ("listenerX")->setValueNotifyingHost (parameters.getParameterRange ("listenerX").convertTo0to1 (roomParam.listenerX));
+                parameters.getParameter ("listenerY")->setValueNotifyingHost (parameters.getParameterRange ("listenerY").convertTo0to1 (roomParam.listenerY));
+                parameters.getParameter ("listenerZ")->setValueNotifyingHost (parameters.getParameterRange ("listenerZ").convertTo0to1 (roomParam.listenerZ));
                 readingSharedParams = false;
             }
             else
@@ -885,12 +766,12 @@ void RoomEncoderAudioProcessor::timerCallback()
         {
             if (roomParam.validReflectionData) {
                 readingSharedParams = true;
-                parameters.getParameterAsValue("reflCoeff").setValue(roomParam.reflCoeff);
-                parameters.getParameterAsValue("numRefl").setValue(roomParam.numRefl);
-                parameters.getParameterAsValue("lowShelfFreq").setValue(roomParam.lowShelfFreq);
-                parameters.getParameterAsValue("lowShelfGain").setValue(roomParam.lowShelfGain);
-                parameters.getParameterAsValue("highShelfFreq").setValue(roomParam.highShelfFreq);
-                parameters.getParameterAsValue("highShelfGain").setValue(roomParam.highShelfGain);
+                parameters.getParameter ("reflCoeff")->setValueNotifyingHost (parameters.getParameterRange ("reflCoeff").convertTo0to1 (roomParam.reflCoeff));
+                parameters.getParameter ("numRefl")->setValueNotifyingHost (parameters.getParameterRange ("numRefl").convertTo0to1 (roomParam.numRefl));
+                parameters.getParameter ("lowShelfFreq")->setValueNotifyingHost (parameters.getParameterRange ("lowShelfFreq").convertTo0to1 (roomParam.lowShelfFreq));
+                parameters.getParameter ("lowShelfGain")->setValueNotifyingHost (parameters.getParameterRange ("lowShelfGain").convertTo0to1 (roomParam.lowShelfGain));
+                parameters.getParameter ("highShelfFreq")->setValueNotifyingHost (parameters.getParameterRange ("highShelfFreq").convertTo0to1 (roomParam.highShelfFreq));
+                parameters.getParameter ("highShelfGain")->setValueNotifyingHost (parameters.getParameterRange ("highShelfGain").convertTo0to1 (roomParam.highShelfGain));
                 readingSharedParams = false;
             }
             else
@@ -959,6 +840,164 @@ void RoomEncoderAudioProcessor::oscMessageReceived (const OSCMessage &message)
     oscParams.processOSCMessage (msg);
 }
 
+void RoomEncoderAudioProcessor::oscBundleReceived (const OSCBundle &bundle)
+{
+    for (int i = 0; i < bundle.size(); ++i)
+    {
+        auto elem = bundle[i];
+        if (elem.isMessage())
+            oscMessageReceived (elem.getMessage());
+        else if (elem.isBundle())
+            oscBundleReceived (elem.getBundle());
+    }
+}
+
+//==============================================================================
+AudioProcessorValueTreeState::ParameterLayout RoomEncoderAudioProcessor::createParameterLayout()
+{
+    // add your audio parameters here
+    std::vector<std::unique_ptr<RangedAudioParameter>> params;
+
+
+
+
+    params.push_back (oscParams.createAndAddParameter ("directivityOrderSetting", "Input Directivity Order", "",
+                                     NormalisableRange<float> (0.0f, 8.0f, 1.0f), 1.0f,
+                                     [](float value) {
+                                         if (value >= 0.5f && value < 1.5f) return "0th";
+                                         else if (value >= 1.5f && value < 2.5f) return "1st";
+                                         else if (value >= 2.5f && value < 3.5f) return "2nd";
+                                         else if (value >= 3.5f && value < 4.5f) return "3rd";
+                                         else if (value >= 4.5f && value < 5.5f) return "4th";
+                                         else if (value >= 5.5f && value < 6.5f) return "5th";
+                                         else if (value >= 6.5f && value < 7.5f) return "6th";
+                                         else if (value >= 7.5f) return "7th";
+                                         else return "Auto"; },
+                                     nullptr));
+
+    params.push_back (oscParams.createAndAddParameter ("inputIsSN3D", "Input Directivity Normalization", "",
+                                     NormalisableRange<float> (0.0f, 1.0f, 1.0f), 1.0f,
+                                     [](float value) { if (value >= 0.5f ) return "SN3D";
+                                         else return "N3D"; },
+                                     nullptr));
+
+    params.push_back (oscParams.createAndAddParameter ("orderSetting", "Output Ambisonics Order", "",
+                                     NormalisableRange<float> (0.0f, 8.0f, 1.0f), 0.0f,
+                                     [](float value) {
+                                         if (value >= 0.5f && value < 1.5f) return "0th";
+                                         else if (value >= 1.5f && value < 2.5f) return "1st";
+                                         else if (value >= 2.5f && value < 3.5f) return "2nd";
+                                         else if (value >= 3.5f && value < 4.5f) return "3rd";
+                                         else if (value >= 4.5f && value < 5.5f) return "4th";
+                                         else if (value >= 5.5f && value < 6.5f) return "5th";
+                                         else if (value >= 6.5f && value < 7.5f) return "6th";
+                                         else if (value >= 7.5f) return "7th";
+                                         else return "Auto"; },
+                                     nullptr));
+    params.push_back (oscParams.createAndAddParameter ("useSN3D", "Normalization", "",
+                                     NormalisableRange<float> (0.0f, 1.0f, 1.0f), 1.0f,
+                                     [](float value) { if (value >= 0.5f ) return "SN3D";
+                                         else return "N3D"; },
+                                     nullptr));
+
+    params.push_back (oscParams.createAndAddParameter ("roomX", "room size x", "m",
+                                     NormalisableRange<float> (1.0f, 30.0f, 0.01f), 10.0f,
+                                     [](float value) { return String(value, 2); }, nullptr));
+    params.push_back (oscParams.createAndAddParameter ("roomY", "room size y", "m",
+                                     NormalisableRange<float> (1.0f, 30.0f, 0.01f), 11.0f,
+                                     [](float value) { return String(value, 2); }, nullptr));
+    params.push_back (oscParams.createAndAddParameter ("roomZ", "room size z", "m",
+                                     NormalisableRange<float> (1.0f, 20.0f, 0.01f), 7.0f,
+                                     [](float value) { return String(value, 2); }, nullptr));
+
+    params.push_back (oscParams.createAndAddParameter ("sourceX", "source position x", "m",
+                                     NormalisableRange<float> (-15.0f, 15.0f, 0.001f), 1.0f,
+                                     [](float value) { return String(value, 3); }, nullptr));
+    params.push_back (oscParams.createAndAddParameter ("sourceY", "source position y", "m",
+                                     NormalisableRange<float> (-15.0f, 15.0f, 0.001f), 1.0f,
+                                     [](float value) { return String(value, 3); }, nullptr));
+    params.push_back (oscParams.createAndAddParameter ("sourceZ", "source position z", "m",
+                                     NormalisableRange<float> (-10.0f, 10.0f, 0.001f), -1.0f,
+                                     [](float value) { return String(value, 3); }, nullptr));
+
+    params.push_back (oscParams.createAndAddParameter ("listenerX", "listener position x", "m",
+                                     NormalisableRange<float> (-15.0f, 15.0f, 0.001f), -1.0f,
+                                     [](float value) { return String(value, 3); }, nullptr));
+    params.push_back (oscParams.createAndAddParameter ("listenerY", "listener position y", "m",
+                                     NormalisableRange<float> (-15.0f, 15.0f, 0.001f), -1.0f,
+                                     [](float value) { return String(value, 3); }, nullptr));
+    params.push_back (oscParams.createAndAddParameter ("listenerZ", "listener position z", "m",
+                                     NormalisableRange<float> (-10.0f, 10.0f, 0.001f), -1.0f,
+                                     [](float value) { return String(value, 3); }, nullptr));
+
+    params.push_back (oscParams.createAndAddParameter ("numRefl", "number of reflections", "",
+                                     NormalisableRange<float> (0.0f, nImgSrc-1, 1.0f), 33.0f,
+                                     [](float value) { return String((int) value); }, nullptr));
+
+    params.push_back (oscParams.createAndAddParameter ("lowShelfFreq", "LowShelf Frequency", "Hz",
+                                     NormalisableRange<float> (20.0f, 20000.0f, 1.0f, 0.2f), 100.0,
+                                     [](float value) { return String((int) value); }, nullptr));
+    params.push_back (oscParams.createAndAddParameter ("lowShelfGain", "LowShelf Gain", "dB",
+                                     NormalisableRange<float> (-15.0f, 5.0f, 0.1f), -5.0f,
+                                     [](float value) { return String(value, 1); }, nullptr));
+
+    params.push_back (oscParams.createAndAddParameter ("highShelfFreq", "HighShelf Frequency", "Hz",
+                                     NormalisableRange<float> (20., 20000.0f, 1.0f, 0.2f), 8000.0,
+                                     [](float value) { return String((int) value); }, nullptr));
+    params.push_back (oscParams.createAndAddParameter ("highShelfGain", "HighShelf Gain", "dB",
+                                     NormalisableRange<float> (-15.0f, 5.0f, 0.1f), -5.0f,
+                                     [](float value) { return String(value, 1); }, nullptr));
+
+    params.push_back (oscParams.createAndAddParameter ("reflCoeff", "Reflection Coefficient", "dB",
+                                     NormalisableRange<float> (-15.0f, 0.0f, 0.01f), -1.0f,
+                                     [](float value) { return String(value, 2); }, nullptr));
+
+    params.push_back (oscParams.createAndAddParameter ("syncChannel", "Synchronize to Channel", "",
+                                     NormalisableRange<float> (0.0f, 4.0f, 1.0f), 0.0f,
+                                     [](float value) {
+                                         if (value >= 0.5f && value < 1.5f) return "Channel 1";
+                                         else if (value >= 1.5f && value < 2.5f) return "Channel 2";
+                                         else if (value >= 2.5f && value < 3.5f) return "Channel 3";
+                                         else if (value >= 3.5f) return "Channel 4";
+                                         else return "None"; },
+                                     nullptr));
+
+    params.push_back (oscParams.createAndAddParameter ("syncRoomSize", "Synchronize Room Dimensions", "",
+                                     NormalisableRange<float> (0.0f, 1.0f, 1.0f), 1.0f,
+                                     [](float value) {
+                                         if (value >= 0.5f) return "YES";
+                                         else return "NO"; },
+                                     nullptr));
+
+    params.push_back (oscParams.createAndAddParameter ("syncReflection", "Synchronize Reflection Properties", "",
+                                     NormalisableRange<float> (0.0f, 1.0f, 1.0f), 1.0f,
+                                     [](float value) {
+                                         if (value >= 0.5f) return "YES";
+                                         else return "NO"; },
+                                     nullptr));
+
+    params.push_back (oscParams.createAndAddParameter ("syncListener", "Synchronize Listener Position", "",
+                                     NormalisableRange<float> (0.0f, 1.0f, 1.0f), 1.0f,
+                                     [](float value) {
+                                         if (value >= 0.5f) return "YES";
+                                         else return "NO"; },
+                                     nullptr));
+
+    params.push_back (oscParams.createAndAddParameter ("renderDirectPath", "Render Direct Path", "",
+                                     NormalisableRange<float> (0.0f, 1.0f, 1.0f), 1.0f,
+                                     [](float value) {
+                                         if (value >= 0.5f) return "YES";
+                                         else return "NO"; },
+                                     nullptr));
+
+
+
+
+
+
+    return { params.begin(), params.end() };
+}
+
 //==============================================================================
 // This creates new instances of the plugin..
 AudioProcessor* JUCE_CALLTYPE createPluginFilter()
