@@ -50,7 +50,7 @@ public:
    #ifndef JucePlugin_PreferredChannelConfigurations
     bool isBusesLayoutSupported (const BusesLayout& layouts) const override;
    #endif
-
+    
     void processBlock (AudioSampleBuffer&, MidiBuffer&) override;
 
     //==============================================================================
@@ -86,9 +86,18 @@ public:
 
     // ====== OSC ======
     void oscMessageReceived (const OSCMessage &message) override;
+    void oscBundleReceived (const OSCBundle &bundle) override;
+
     OSCReceiverPlus& getOSCReceiver () { return oscReceiver; }
     // =================
 
+    //======= Parameters ===========================================================
+    AudioProcessorValueTreeState::ParameterLayout createParameterLayout();
+    //==============================================================================
+
+    inline void updateQuaternions();
+    inline void updateEuler();
+
     Vector3D<float> posC, posL, posR;
 
     Atomic<bool> updatedPositionData;
@@ -113,17 +122,23 @@ public:
 
 private:
     //==============================================================================
+    OSCParameterInterface oscParams;
+    OSCReceiverPlus oscReceiver;
+    AudioProcessorValueTreeState parameters;
+
     bool processorUpdatingParams;
 
-    AudioProcessorValueTreeState parameters;
-    OSCParameterInterface oscParameterInterface;
-    OSCReceiverPlus oscReceiver;
+
 
     float SHL[64];
     float SHR[64];
     float _SHL[64];
     float _SHR[64];
 
+    Atomic<bool> positionHasChanged = true;
+
+    iem::Quaternion<float> quaternionDirection;
+    
     AudioBuffer<float> bufferCopy;
 
     LinearSmoothedValue<float> smoothAzimuthL, smoothElevationL;
