@@ -30,19 +30,20 @@ class SpherePannerBackground :  public Component
 {
 
 public:
-    SpherePannerBackground() : Component() {
+    SpherePannerBackground() : Component()
+    {
         setBufferedToImage(true);
     };
     ~SpherePannerBackground() {};
 
     void resized() override
     {
-        const Rectangle<float> sphere(getLocalBounds().reduced(10, 10).toFloat());
+        const Rectangle<float> sphere (getLocalBounds().reduced (10, 10).toFloat());
 
-        radius = 0.5f * jmin(sphere.getWidth(), sphere.getHeight());
+        radius = 0.5f * jmin (sphere.getWidth(), sphere.getHeight());
         centre = getLocalBounds().getCentre();
-        sphereArea.setBounds(0, 0, 2*radius, 2*radius);
-        sphereArea.setCentre(centre.toFloat());
+        sphereArea.setBounds (0, 0, 2 * radius, 2 * radius);
+        sphereArea.setCentre (centre.toFloat());
     };
 
     void paint (Graphics& g) override
@@ -50,49 +51,48 @@ public:
         const Rectangle<float> bounds = getLocalBounds().toFloat();
         const float centreX = bounds.getCentreX();
         const float centreY = bounds.getCentreY();
-        const Rectangle<float> sphere(bounds.reduced(10, 10));
 
-        g.setColour(Colours::white);
-        g.drawEllipse(centreX-radius, centreY - radius, 2.0f * radius, 2.0f * radius, 1.0f);
+        g.setColour (Colours::white);
+        g.drawEllipse (centreX-radius, centreY - radius, 2.0f * radius, 2.0f * radius, 1.0f);
 
-        g.setFont(getLookAndFeel().getTypefaceForFont (Font(12.0f, 1)));
-        g.setFont(12.0f);
-        g.drawText("FRONT", centreX-15, centreY-radius-12, 30, 12, Justification::centred);
-        g.drawText("BACK", centreX-15, centreY+radius, 30, 12, Justification::centred);
-        g.drawFittedText("L\nE\nF\nT", sphereArea.getX()-10, centreY-40, 10, 80, Justification::centred, 4);
-        g.drawFittedText("R\nI\nG\nH\nT", sphereArea.getRight(), centreY-40, 10, 80, Justification::centred, 5);
+        g.setFont (getLookAndFeel().getTypefaceForFont (Font (12.0f, 1)));
+        g.setFont (12.0f);
+        g.drawText ("FRONT", centreX-15, centreY-radius - 12, 30, 12, Justification::centred);
+        g.drawText ("BACK", centreX-15, centreY+radius, 30, 12, Justification::centred);
+        g.drawFittedText ("L\nE\nF\nT", sphereArea.getX() - 10, centreY - 40, 10, 80, Justification::centred, 4);
+        g.drawFittedText ("R\nI\nG\nH\nT", sphereArea.getRight(), centreY - 40, 10, 80, Justification::centred, 5);
 
-        g.setColour(Colours::steelblue.withMultipliedAlpha(0.2f));
+        g.setColour (Colours::steelblue.withMultipliedAlpha (0.2f));
         Path circles;
 
         for (int deg = 75; deg >= 0; deg -= 15)
         {
             float rCirc;
             if (! linearElevation)
-                rCirc = radius * std::cos (Conversions<float>::degreesToRadians(deg));
+                rCirc = radius * std::cos (Conversions<float>::degreesToRadians (deg));
             else
                 rCirc = radius * (90 - deg) / 90;
             circles.addEllipse(centreX - rCirc, centreY - rCirc, 2.0f * rCirc, 2.0f * rCirc);
             g.fillPath(circles);
         }
 
-        g.setColour(Colours::Colours::steelblue.withMultipliedAlpha(0.7f));
-        g.strokePath(circles, PathStrokeType(.5f));
+        g.setColour (Colours::Colours::steelblue.withMultipliedAlpha (0.7f));
+        g.strokePath (circles, PathStrokeType (.5f));
 
-        ColourGradient gradient(Colours::black.withMultipliedAlpha(0.7f), centreX, centreY, Colours::black.withMultipliedAlpha(0.1f), 0, 0, true);
-        g.setGradientFill(gradient);
+        ColourGradient gradient(Colours::black.withMultipliedAlpha (0.7f), centreX, centreY, Colours::black.withMultipliedAlpha (0.1f), 0, 0, true);
+        g.setGradientFill (gradient);
 
         Path line;
-        line.startNewSubPath(centreX, centreY-radius);
-        line.lineTo(centreX, centreY+radius);
+        line.startNewSubPath (centreX, centreY - radius);
+        line.lineTo (centreX, centreY + radius);
 
         Path path;
-        path.addPath(line);
-        path.addPath(line, AffineTransform().rotation(0.25f*M_PI, centreX, centreY));
-        path.addPath(line, AffineTransform().rotation(0.5f*M_PI, centreX, centreY));
-        path.addPath(line, AffineTransform().rotation(0.75f*M_PI, centreX, centreY));
+        path.addPath (line);
+        path.addPath (line, AffineTransform().rotation (0.25f * MathConstants<float>::pi, centreX, centreY));
+        path.addPath (line, AffineTransform().rotation (0.5f * MathConstants<float>::pi, centreX, centreY));
+        path.addPath (line, AffineTransform().rotation (0.75f * MathConstants<float>::pi, centreX, centreY));
 
-        g.strokePath(path, PathStrokeType(0.5f));
+        g.strokePath(path, PathStrokeType (0.5f));
     }
     void setElevationStyle (bool linear) { linearElevation = linear; };
 
@@ -108,12 +108,14 @@ private:
 class  SpherePanner :  public Component
 {
 public:
-    SpherePanner() : Component() {
-        setBufferedToImage(true);
+    SpherePanner() : Component()
+    {
+        setBufferedToImage (true);
 
-        addAndMakeVisible(background);
-        background.addMouseListener(this, false); // could this be risky?
+        addAndMakeVisible (background);
+        background.addMouseListener (this, false); // could this be risky?
     };
+
     ~SpherePanner() {};
 
 
@@ -143,17 +145,17 @@ public:
         void setActive (bool isActive) { active = isActive; }
         bool isActive() { return active; }
 
-        void setColour( Colour newColour) { colour = newColour; }
-        void setTextColour( Colour newColour) { textColour = newColour; }
+        void setColour (Colour newColour) { colour = newColour; }
+        void setTextColour (Colour newColour) { textColour = newColour; }
         Colour getColour() { return colour; }
         Colour getTextColour() { return textColour; }
 
         void setLabel(String newLabel) {label = newLabel;}
 
-        void setGrabPriority(int newPriority) {grabPriority = newPriority;}
+        void setGrabPriority (int newPriority) { grabPriority = newPriority; }
         int getGrabPriority() {return grabPriority;}
-        void setGrabRadius(float newRadius) {grabRadius = newRadius;}
-        float getGrabRadius() {return grabRadius;}
+        void setGrabRadius (float newRadius) { grabRadius = newRadius; }
+        float getGrabRadius() { return grabRadius; }
 
 
         String getLabel() {return label;};
@@ -177,20 +179,21 @@ public:
         void moveElement (const MouseEvent &event, Point<int> centre, float radius, bool upBeforeDrag, bool linearElevation, bool rightClick) override
         {
             Point<int> pos = event.getPosition();
-            const float azimuth = -1.0f * centre.getAngleToPoint(pos);
-            float r = centre.getDistanceFrom(pos) / radius;
-            if (r > 1.0f) {
+            const float azimuth = -1.0f * centre.getAngleToPoint (pos);
+            float r = centre.getDistanceFrom (pos) / radius;
+            if (r > 1.0f)
+            {
                 r = 1.0f / r;
                 upBeforeDrag = !upBeforeDrag;
             }
 
             if (linearElevation)
-                r = std::sin(r * 1.570796327f);
+                r = std::sin (r * 1.570796327f);
 
 
-            float elevation = std::acos(r);
+            float elevation = std::acos (r);
             if (! upBeforeDrag) elevation *= -1.0f;
-            position = Conversions<float>::sphericalToCartesian(azimuth, elevation);
+            position = Conversions<float>::sphericalToCartesian (azimuth, elevation);
         }
 
         /*
@@ -201,7 +204,7 @@ public:
             return position;
         };
 
-        bool setCoordinates(Vector3D<float> newPosition) // is true when position is updated (use it for repainting)
+        bool setCoordinates (Vector3D<float> newPosition) // is true when position is updated (use it for repainting)
         {
             if (position.x != newPosition.x || position.y != newPosition.y || position.z != newPosition.z)
             {
@@ -219,7 +222,7 @@ public:
     class AzimuthElevationParameterElement : public Element
     {
     public:
-        AzimuthElevationParameterElement(AudioProcessorParameter& azimuthParameter, NormalisableRange<float> azimuthParameterRange, AudioProcessorParameter& elevationParameter, NormalisableRange<float> elevationParameterRange) : Element(), azimuth(azimuthParameter), azimuthRange(azimuthParameterRange), elevation(elevationParameter), elevationRange(elevationParameterRange) {}
+        AzimuthElevationParameterElement (AudioProcessorParameter& azimuthParameter, NormalisableRange<float> azimuthParameterRange, AudioProcessorParameter& elevationParameter, NormalisableRange<float> elevationParameterRange) : Element(), azimuth (azimuthParameter), azimuthRange (azimuthParameterRange), elevation (elevationParameter), elevationRange (elevationParameterRange) {}
 
         void startMovement() override
         {
@@ -231,7 +234,7 @@ public:
         {
             Point<int> pos = event.getPosition();
             const float azi = -1.0f * centre.getAngleToPoint(pos);
-            const float azimuthInDegrees { Conversions<float>::radiansToDegrees(azi) };
+            const float azimuthInDegrees { Conversions<float>::radiansToDegrees (azi) };
 
             if (! rightClick)
             {
@@ -245,15 +248,15 @@ public:
 
                 if (linearElevation)
                     r = std::sin(r * 1.570796327f);
-                float ele = std::acos(r);
+                float ele = std::acos (r);
                 if (! upBeforeDrag) ele *= -1.0f;
 
-                const float elevationInDegrees { Conversions<float>::radiansToDegrees(ele) };
+                const float elevationInDegrees { Conversions<float>::radiansToDegrees (ele) };
 
-                elevation.setValueNotifyingHost(elevationRange.convertTo0to1(elevationInDegrees));
+                elevation.setValueNotifyingHost (elevationRange.convertTo0to1 (elevationInDegrees));
             }
 
-            azimuth.setValueNotifyingHost(azimuthRange.convertTo0to1(azimuthInDegrees));
+            azimuth.setValueNotifyingHost (azimuthRange.convertTo0to1 (azimuthInDegrees));
         }
 
         void stopMovement() override
@@ -264,14 +267,14 @@ public:
 
         const float getAzimuthInRadians()
         {
-            const float azimuthInDegrees {azimuthRange.convertFrom0to1(azimuth.getValue())};
-            return Conversions<float>::degreesToRadians(azimuthInDegrees);
+            const float azimuthInDegrees { azimuthRange.convertFrom0to1 (azimuth.getValue()) };
+            return Conversions<float>::degreesToRadians (azimuthInDegrees);
         }
 
         const float getElevationInRadians()
         {
-            const float elevationInDegrees {elevationRange.convertFrom0to1(elevation.getValue())};
-            return Conversions<float>::degreesToRadians(elevationInDegrees);
+            const float elevationInDegrees { elevationRange.convertFrom0to1 (elevation.getValue()) };
+            return Conversions<float>::degreesToRadians (elevationInDegrees);
         }
 
         /*
@@ -279,7 +282,7 @@ public:
          */
         Vector3D<float> getCoordinates() override
         {
-            return Conversions<float>::sphericalToCartesian(getAzimuthInRadians(), getElevationInRadians());
+            return Conversions<float>::sphericalToCartesian (getAzimuthInRadians(), getElevationInRadians());
         };
 
     private:
@@ -292,7 +295,7 @@ public:
     class RollWidthParameterElement : public Element
     {
     public:
-        RollWidthParameterElement(AzimuthElevationParameterElement& center, AudioProcessorParameter& rollParameter, NormalisableRange<float> rollParameterRange, AudioProcessorParameter& widthParameter, NormalisableRange<float> widthParameterRange) : Element(), centerElement(center), roll(rollParameter), rollRange(rollParameterRange), width(widthParameter), widthRange(widthParameterRange) {}
+        RollWidthParameterElement(AzimuthElevationParameterElement& center, AudioProcessorParameter& rollParameter, NormalisableRange<float> rollParameterRange, AudioProcessorParameter& widthParameter, NormalisableRange<float> widthParameterRange) : Element(), centerElement (center), roll (rollParameter), rollRange(rollParameterRange), width (widthParameter), widthRange (widthParameterRange) {}
 
         void startMovement() override
         {
@@ -303,25 +306,26 @@ public:
         void moveElement (const MouseEvent &event, Point<int> centre, float radius, bool upBeforeDrag, bool linearElevation, bool rightClick) override
         {
             Point<int> pos = event.getPosition();
-            const float azi = -1.0f * centre.getAngleToPoint(pos);
+            const float azi = -1.0f * centre.getAngleToPoint (pos);
             float r = centre.getDistanceFrom(pos) / radius;
-            if (r > 1.0f) {
+            if (r > 1.0f)
+            {
                 r = 1.0f / r;
                 upBeforeDrag = !upBeforeDrag;
             }
 
             if (linearElevation)
-                r = std::sin(r * 1.570796327f);
+                r = std::sin (r * 1.570796327f);
 
-            float ele = std::acos(r);
+            float ele = std::acos (r);
             if (! upBeforeDrag) ele *= -1.0f;
 
-            Vector3D<float> posXYZ {Conversions<float>::sphericalToCartesian(azi, ele)};
+            Vector3D<float> posXYZ {Conversions<float>::sphericalToCartesian (azi, ele)};
 
             // ==== calculate width
             Vector3D<float> dPos = posXYZ - centerElement.getCoordinates();
-            const float alpha = 4.0f*asinf(dPos.length()/2.0f);
-            width.setValueNotifyingHost(widthRange.convertTo0to1(Conversions<float>::radiansToDegrees(alpha)));
+            const float alpha = 4.0f * std::asin (dPos.length() / 2.0f);
+            width.setValueNotifyingHost (widthRange.convertTo0to1 (Conversions<float>::radiansToDegrees (alpha)));
 
             // ==== calculate roll
             iem::Quaternion<float> quat;
@@ -340,10 +344,10 @@ public:
 
             quat.rotateVector(xyz, xyz);
 
-            float rollInRadians = atan2(xyz[2], xyz[1]);
-            if (isMirrored) rollInRadians = atan2(-xyz[2], -xyz[1]);
+            float rollInRadians = atan2 (xyz[2], xyz[1]);
+            if (isMirrored) rollInRadians = atan2 (-xyz[2], -xyz[1]);
 
-            roll.setValueNotifyingHost(rollRange.convertTo0to1(Conversions<float>::radiansToDegrees(rollInRadians)));
+            roll.setValueNotifyingHost (rollRange.convertTo0to1 (Conversions<float>::radiansToDegrees (rollInRadians)));
         }
 
         void stopMovement() override
@@ -360,13 +364,13 @@ public:
             float ypr[3];
             ypr[0] = centerElement.getAzimuthInRadians();
             ypr[1] = - centerElement.getElevationInRadians(); // pitch
-            ypr[2] = Conversions<float>::degreesToRadians(rollRange.convertFrom0to1(roll.getValue()));
+            ypr[2] = Conversions<float>::degreesToRadians (rollRange.convertFrom0to1 (roll.getValue()));
 
             //updating not active params
             iem::Quaternion<float> quat;
             quat.fromYPR(ypr);
 
-            const float widthInRadiansQuarter {Conversions<float>::degreesToRadians(widthRange.convertFrom0to1(width.getValue())) / 4.0f};
+            const float widthInRadiansQuarter (Conversions<float>::degreesToRadians(widthRange.convertFrom0to1 (width.getValue())) / 4.0f);
 
             iem::Quaternion<float> quatLRot {iem::Quaternion<float>(cos(widthInRadiansQuarter), 0.0f, 0.0f, sin(widthInRadiansQuarter))};
             if (isMirrored)
@@ -375,7 +379,7 @@ public:
             iem::Quaternion<float> quatL = quat * quatLRot;
 
             float xyz[3];
-            quatL.toCartesian(xyz);
+            quatL.toCartesian (xyz);
 
             return Vector3D<float> (xyz[0], xyz[1], xyz[2]);
         };
@@ -406,7 +410,6 @@ public:
         const Rectangle<float> bounds = getLocalBounds().toFloat();
         const float centreX = bounds.getCentreX();
         const float centreY = bounds.getCentreY();
-        const Rectangle<float> sphere(bounds.reduced(10, 10));
 
         Path panPos;
         const int size = elements.size();
@@ -426,12 +429,12 @@ public:
             }
 
             Rectangle<float> temp(centreX - pos.y * radius - diam / 2, centreY - pos.x * radius - diam / 2, diam, diam);
-            panPos.addEllipse(temp);
-            g.strokePath(panPos, PathStrokeType(1.0f));
-            g.setColour((handle->isActive() ? handle->getColour() : Colours::grey).withMultipliedAlpha(pos.z >= -0.0f ? 1.0f : 0.4f));
-            g.fillPath(panPos);
-            g.setColour(handle->getTextColour());
-            g.drawFittedText(handle->getLabel(), temp.toNearestInt(), Justification::centred, 1);
+            panPos.addEllipse (temp);
+            g.strokePath (panPos, PathStrokeType(1.0f));
+            g.setColour ((handle->isActive() ? handle->getColour() : Colours::grey).withMultipliedAlpha (pos.z >= -0.0f ? 1.0f : 0.4f));
+            g.fillPath (panPos);
+            g.setColour (handle->getTextColour());
+            g.drawFittedText (handle->getLabel(), temp.toNearestInt(), Justification::centred, 1);
             panPos.clear();
         }
     };
@@ -448,8 +451,8 @@ public:
         int oldActiveElem = activeElem;
         activeElem = -1;
 
-        const float centreX = 0.5* (float)getBounds().getWidth();
-        const float centreY = 0.5* (float)getBounds().getHeight();
+        const float centreX = 0.5f * (float) getBounds().getWidth();
+        const float centreY = 0.5f * (float) getBounds().getHeight();
 
         int nElem = elements.size();
 
@@ -466,13 +469,13 @@ public:
 
             float tx,ty;
             for (int i = elements.size(); --i >= 0;) {
-                Element* handle(elements.getUnchecked (i));
+                Element* handle (elements.getUnchecked (i));
                 Vector3D<float> pos = handle->getCoordinates();
 
                 if (linearElevation)
                 {
-                    const float r = sqrt(pos.y * pos.y + pos.x * pos.x);
-                    const float factor = std::asin(r) / r / 1.570796327f; // pi / 2
+                    const float r = sqrt (pos.y * pos.y + pos.x * pos.x);
+                    const float factor = std::asin (r) / r / 1.570796327f; // pi / 2
                     pos *= factor;
                 }
 
@@ -481,11 +484,13 @@ public:
                 dist[i] = tx*tx + ty*ty;
 
                 if (dist[i] <= handle->getGrabRadius()) {
-                    if (handle->getGrabPriority() > highestPriority) {
+                    if (handle->getGrabPriority() > highestPriority)
+                    {
                         activeElem = i;
                         highestPriority = handle->getGrabPriority();
                     }
-                    else if (handle->getGrabPriority() == highestPriority && dist[i] < dist[activeElem]) {
+                    else if (handle->getGrabPriority() == highestPriority && dist[i] < dist[activeElem])
+                    {
                         activeElem = i;
                     }
                 }
@@ -508,16 +513,14 @@ public:
 
     void mouseDown (const MouseEvent &event) override
     {
-        if (activeElem != -1) {
-            elements.getUnchecked(activeElem)->startMovement();
-        }
+        if (activeElem != -1)
+            elements.getUnchecked (activeElem)->startMovement();
     }
 
     void mouseUp (const MouseEvent &event) override
     {
-        if (activeElem != -1) {
-            elements.getUnchecked(activeElem)->stopMovement();
-        }
+        if (activeElem != -1)
+            elements.getUnchecked (activeElem)->stopMovement();
     }
 
     void mouseDoubleClick (const MouseEvent &event) override
@@ -527,30 +530,35 @@ public:
         repaint();
     }
 
-    void addListener (Listener* const listener) {
+    void addListener (Listener* const listener)
+    {
         jassert (listener != nullptr);
         if (listener != nullptr)
             listeners.add (listener);
     };
 
-    void removeListener (Listener* const listener) {
-        listeners.removeFirstMatchingValue(listener);
+    void removeListener (Listener* const listener)
+    {
+        listeners.removeFirstMatchingValue (listener);
     };
 
-    void addElement (Element* const element) {
+    void addElement (Element* const element)
+    {
         jassert (element != nullptr);
         if (element !=0)
             elements.add (element);
     };
-    void removeElement (Element* const element) {
-        elements.removeFirstMatchingValue(element);
+
+    void removeElement (Element* const element)
+    {
+        elements.removeFirstMatchingValue (element);
     };
 
-    int indexofSmallestElement(float *array, int size)
+    int indexofSmallestElement (float *array, int size)
     {
         int index = 0;
 
-        for(int i = 1; i < size; i++)
+        for (int i = 1; i < size; i++)
         {
             if(array[i] < array[index])
                 index = i;
@@ -559,7 +567,8 @@ public:
         return index;
     }
 
-    void setElevationStyle (bool linear) {
+    void setElevationStyle (bool linear)
+    {
         linearElevation = linear;
         background.setElevationStyle(linear);
     };
@@ -568,7 +577,7 @@ private:
     float radius = 1.0f;
     Rectangle<float> sphereArea;
     Point<int> centre;
-    int activeElem;
+    int activeElem = -1;
     bool activeElemWasUpBeforeDrag;
     Array<Listener*> listeners;
     Array<Element*> elements;
